@@ -5,12 +5,12 @@ use syn::ItemStruct;
 use crate::shared::StructInfo;
 
 fn generate_header(mut i: StructInfo) -> TokenStream2 {
-    let _ = i.pop_field();
+    i.pop_field();
 
     let (struct_name, generics, generics_without_bounds, _, field_types, _, _) = i.into_tuple();
 
     quote! {
-        impl#generics neli::Header for #struct_name#generics_without_bounds {
+        impl #generics neli::Header for #struct_name #generics_without_bounds {
             fn header_size() -> usize {
                 #( <#field_types as neli::TypeSize>::type_size() )+*
             }
